/*
 * @lket@ignore-file -- We actually don't want this file for upstream :)
 *
 * WiMAX kernel stack
 * Compatibility glue
 *
 *
 * Copyright (C) 2008 Intel Corporation <linux-wimax@intel.com>
 * Inaky Perez-Gonzalez <inaky.perez-gonzalez@intel.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include <config.h>
#include <linux/version.h>

#if defined(NEED_GENL_MC_GROUPS)

/*
 * This is a "fake" named multicast groups implementation, for compat
 * with 2.6.22.
 *
 * As 'struct genl_family' lacks a multicast group list, we add it to
 * a list head. We create a structure for describing multicast groups.
 *
 * This is very hacky, but it does the job.
 */
#include <net/wimax.h>
#include <linux/mutex.h>
#include "wimax-internal.h"

static LIST_HEAD(mc_group_list);
static DEFINE_MUTEX(mc_group_mutex);

int genl_register_mc_group(struct genl_family *genl_family,
			   struct genl_multicast_group *mc_group)
{
	unsigned cnt = 0;
	struct list_head *itr;

	mutex_lock(&mc_group_mutex);
	list_add(&mc_group->list, &mc_group_list);
	list_for_each(itr, &mc_group_list)
		cnt++;
	mc_group->id = cnt;
	mutex_unlock(&mc_group_mutex);
	return 0;
}

void genl_unregister_mc_group(struct genl_family *genl_family,
			      struct genl_multicast_group *mc_group)
{
	mutex_lock(&mc_group_mutex);
	list_del(&mc_group->list);
	mutex_unlock(&mc_group_mutex);
}

#endif /* #if defined(NEED_GENL_MC_GROUPS) */


#if defined(NEED_DEBUGFS_REMOVE_RECURSIVE)
#include <linux/module.h>
#include <linux/debugfs.h>
/*
 * debugfs_remove_recursive was introduced in 2.6.27
 *
 * It's almost impossible to reimplement it externally, so we don't
 * attempt to. By removing the toplevel entry we hope that at least it
 * gets disconnected from the top level directory tree and it won't
 * cause any bad side effects.
 *
 * Still it might probably leave dangling counts in the debugfs
 * filesystem.
 *
 * This sucks--feel free to submit a better one.
 *
 * UNTESTED!
 */
#warning FIXME: debugfs_remove_recursive() untested!
void debugfs_remove_recursive(struct dentry *dentry)
{
	debugfs_remove(dentry);
	printk(KERN_ERR "WARNING: will leak a few dentries\n");
}
EXPORT_SYMBOL_GPL(debugfs_remove_recursive);

#endif /* #if defined(NEED_DEBUGFS_REMOVE_RECURSIVE) */


#if defined(NEED_DEBUGFS_CREATE_SIZE_T)

#include <linux/module.h>
#include <linux/debugfs.h>

/* Do debugfs simple attributes with size_t */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
static int debugfs_size_t_set(void *data, u64 val)
{
	*(size_t *)data = val;
	return 0;
}
static int debugfs_size_t_get(void *data, u64 *val)
{
	*val = *(size_t *)data;
	return 0;
}
#else /* < 2.6.25 */
static void debugfs_size_t_set(void *data, u64 val)
{
	*(size_t *)data = val;
}
static u64 debugfs_size_t_get(void *data)
{
	return *(size_t *)data;
}
#endif
DEFINE_SIMPLE_ATTRIBUTE(fops_size_t, debugfs_size_t_get, debugfs_size_t_set,
			"%llu\n");

struct dentry *debugfs_create_size_t(const char *name, mode_t mode,
				     struct dentry *parent, size_t *value)
{
	return debugfs_create_file(name, mode, parent, value, &fops_size_t);
}
EXPORT_SYMBOL_GPL(debugfs_create_size_t);

#endif /* #if defined(NEED_DEBUGFS_CREATE_SIZE_T) */
